# PyGME is a Python component generator for the Generic Modeling Environment (GME).

# Copyright (c) 2011, Institute for Software Integrated Systems, Vanderbilt University
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose, without fee, and without written agreement is
# hereby granted, provided that the above copyright notice, the following
# two paragraphs and the author appear in all copies of this software.
#
# IN NO EVENT SHALL THE VANDERBILT UNIVERSITY BE LIABLE TO ANY PARTY FOR
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
# OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE VANDERBILT
# UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# THE VANDERBILT UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
# ON AN "AS IS" BASIS, AND THE VANDERBILT UNIVERSITY HAS NO OBLIGATION TO
# PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
#
# Author: Larry Howard

import os.path, sys

class PyGME_Generator(object):

    def __init__(self, baseFolder):
        [setattr(self, x, None) for x in ('specPath', 'genFolder', 'registry', 'componentPath')]
        self.baseFolder = baseFolder
        self.register = False
        self.defaultSpecification = "component.xml"
        self.specifications = {'tooltip' : '', 'iconpath' : '', 'clsid' : self.generateUUID() }

    def processCommandLine(self):
        # process registration
        for arg in sys.argv:
            if arg.startswith('-register'):
                self.register = True
                sarg = arg.split('=')
                self.registry = len(sarg) == 2 and sarg[1] == 'system' and 2 or 1
                sys.argv.remove(arg)
                break
        # process component specification
        self.specPath = len(sys.argv) > 1 and os.path.abspath(sys.argv[1]) or os.path.join(self.baseFolder, self.defaultSpecification)
        if not (os.path.exists(self.specPath) and os.path.isfile(self.specPath)):
            print "Component specification file path '%s' does not exist or is not a file." % self.specPath
            raise Exception()
        self.genFolder = os.path.dirname(self.specPath)

    def processComponentSpecification(self):
        specDict = self.specifications
        from xml.dom.minidom import parse, Element
        try:
            specXML = parse(os.path.join(self.specPath))
        except Exception, e:
            print "Error parsing XML specification file: %s." % e
            raise
        de = specXML.documentElement
        for attr in ('name', 'version', 'type', 'paradigm'):
            specDict[attr] = getattr(de.attributes.get(attr), 'value', None)
            if specDict[attr] is None:
                print "Attribute '%s' is missing from 'component' tag."
            if attr == 'type' and specDict[attr] and specDict[attr] not in ('Interpreter', 'Addon'):
                print "Unrecognized component type.  Must be either 'Interpreter' or 'Addon'."
                specDict[attr] = None
        if None in specDict.values():
            print "Terminating due to component specification errors."
            raise Exception()
        # optional specifications
        if specDict['type'] == 'Interpreter':
            for c in de.childNodes:
                if isinstance(c, Element):
                    specDict[c.nodeName] = getattr(c.attributes.get('value'), 'value', None)
            if 'iconpath' in specDict:
                iconPath = os.path.normpath(specDict['iconpath'].replace('\\', os.path.sep))
                if not os.path.isabs(iconPath):
                    iconPath = os.path.join(self.baseFolder, iconPath)
                if not (os.path.exists(iconPath) and os.path.isfile(iconPath)):
                    print "Component icon path '%s' does not exist or is not a file." % iconPath
                    raise Exception()
                specDict['icondir'], specDict['iconpath'] = os.path.split(iconPath)

    def generateUUID(self):
        try:
            import pythoncom
            generator = pythoncom.CreateGuid
        except Exception, e:
            print "Unable to import pythoncom module. Your Python installation appears to lack the required 'Win32 Extensions for Python' package."
            if  self.register:
                self.register = False
                print "Unable to register GME COM components."
            try:
                import uuid
                generator = lambda : str(uuid.uuid1()).upper()
                print "Generating component using built-in Python uuid module in place of pythoncom.CreateGuid."
            except Exception,e:
                "Unable to generate component. No available UUID generator."
                raise
        return generator()

    def generateComponent(self):
        if self.specifications['name'].find('.') != -1:
            print "Component name should not contain '.'"
            raise ValueError()
        try:
            from Generator import InterpreterTemplate, AddonTemplate
        except Exception,e:
            print "Unable to import component templates.  Not found relative to PyGME.py or on PythonPath."
            raise
        template = self.specifications['type'] == 'Interpreter' and InterpreterTemplate or AddonTemplate
        self.componentPath = os.path.join(self.genFolder, "%s.py" % self.specifications['name'])
        f = file(self.componentPath, "w")
        f.write(template % self.specifications)
        f.close()

    def registerComponent(self):
        if self.register:
            if self.baseFolder not in sys.path:
                sys.path.insert(0, self.baseFolder)
            mod = __import__(self.specifications['name'])
            comp = getattr(mod, self.specifications['name'])
            comp.RegisterSelf(self.registry, self.specifications['icondir'])

def generate():
    print "PyGME. Starting generation..."
    try:
        generator = PyGME_Generator(baseFolder=os.path.dirname(os.path.abspath(__file__)))
        generator.processCommandLine()
        generator.processComponentSpecification()
        generator.generateComponent()
        generator.registerComponent()
        print "PyGME. Component generation complete.  Your generated component is in '%s'." % generator.componentPath
    except Exception,e:
        print "PyGME. Component generation failed."

if __name__ == "__main__":
    generate()